@@ -72,6 +72,15 @@ class raii_tmpdir:
 def short_tmpdir():
     return raii_tmpdir()
 
+def readdir_inode(dir):
+    cmd = base_cmdline + [ pjoin(basename, 'test', 'readdir_inode'), dir ]
+    with subprocess.Popen(cmd, stdout=subprocess.PIPE,
+                          universal_newlines=True) as proc:
+        lines = proc.communicate()[0].splitlines()
+    lines.sort()
+    return lines
+
+
 @pytest.mark.parametrize("cmdline_builder", (invoke_directly, invoke_mount_fuse,
                                              invoke_mount_fuse_drop_privileges))
 @pytest.mark.parametrize("options", powerset(options))
@@ -143,8 +152,8 @@ def test_passthrough(short_tmpdir, name, debug, output_checker, writeback):
         tst_append(src_dir, work_dir)
         tst_seek(src_dir, work_dir)
         tst_mkdir(work_dir)
-        tst_rmdir(src_dir, work_dir)
-        tst_unlink(src_dir, work_dir)
+        tst_rmdir(work_dir, src_dir)
+        tst_unlink(work_dir, src_dir)
         tst_symlink(work_dir)
         if os.getuid() == 0:
             tst_chown(work_dir)
@@ -199,8 +208,14 @@ def test_passthrough_hp(short_tmpdir, cache, output_checker):
         tst_append(src_dir, mnt_dir)
         tst_seek(src_dir, mnt_dir)
         tst_mkdir(mnt_dir)
-        tst_rmdir(src_dir, mnt_dir)
-        tst_unlink(src_dir, mnt_dir)
+        if cache:
+            # if cache is enabled, no operations should go through
+            # src_dir as the cache will become stale.
+            tst_rmdir(mnt_dir)
+            tst_unlink(mnt_dir)
+        else:
+            tst_rmdir(mnt_dir, src_dir)
+            tst_unlink(mnt_dir, src_dir)
         tst_symlink(mnt_dir)
         if os.getuid() == 0:
             tst_chown(mnt_dir)
@@ -387,10 +402,13 @@ def os_open(name, flags):
 def os_create(name):
     os.close(os.open(name, os.O_CREAT | os.O_RDWR))
 
-def tst_unlink(src_dir, mnt_dir):
+def tst_unlink(mnt_dir, src_dir=None):
     name = name_generator()
     fullname = mnt_dir + "/" + name
-    with open(pjoin(src_dir, name), 'wb') as fh:
+    srcname = fullname
+    if src_dir is not None:
+        srcname = pjoin(src_dir, name)
+    with open(srcname, 'wb') as fh:
         fh.write(b'hello')
     assert name in os.listdir(mnt_dir)
     os.unlink(fullname)
@@ -410,10 +428,13 @@ def tst_mkdir(mnt_dir):
     assert fstat.st_nlink in (1,2)
     assert dirname in os.listdir(mnt_dir)
 
-def tst_rmdir(src_dir, mnt_dir):
+def tst_rmdir(mnt_dir, src_dir=None):
     name = name_generator()
     fullname = mnt_dir + "/" + name
-    os.mkdir(pjoin(src_dir, name))
+    srcname = fullname
+    if src_dir is not None:
+        srcname = pjoin(src_dir, name)
+    os.mkdir(srcname)
     assert name in os.listdir(mnt_dir)
     os.rmdir(fullname)
     with pytest.raises(OSError) as exc_info:
@@ -590,6 +611,10 @@ def tst_readdir(src_dir, mnt_dir):
     listdir_should.sort()
     assert listdir_is == listdir_should
 
+    inodes_is = readdir_inode(mnt_newdir)
+    inodes_should = readdir_inode(src_newdir)
+    assert inodes_is == inodes_should
+
     os.unlink(file_)
     os.unlink(subfile)
     os.rmdir(subdir)
@@ -611,6 +636,10 @@ def tst_readdir_big(src_dir, mnt_dir):
     listdir_should = sorted(os.listdir(src_dir))
     assert listdir_is == listdir_should
 
+    inodes_is = readdir_inode(mnt_dir)
+    inodes_should = readdir_inode(src_dir)
+    assert inodes_is == inodes_should
+
     for fname in fnames:
         stat_src = os.stat(pjoin(src_dir, fname))
         stat_mnt = os.stat(pjoin(mnt_dir, fname))
@@ -619,7 +648,7 @@ def tst_readdir_big(src_dir, mnt_dir):
         assert stat_src.st_ctime == stat_mnt.st_ctime
         assert stat_src.st_size == stat_mnt.st_size
         os.unlink(pjoin(src_dir, fname))
-    
+
 def tst_truncate_path(mnt_dir):
     assert len(TEST_DATA) > 1024
 
